unit MainUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, StdCtrls,
  ExtCtrls, ComCtrls;

const
  AM_REPORTDATA = WM_USER + $71;

type
  TMainForm = class(TForm)
    Panel1: TPanel;
    LogMemo: TMemo;
    StartButton: TButton;
    StopButton: TButton;
    ReportRadios: TRadioGroup;
    procedure FormCreate(Sender: TObject);
    procedure StartButtonClick(Sender: TObject);
    procedure StopButtonClick(Sender: TObject);
    procedure ReportRadiosClick(Sender: TObject);
  protected
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

uses
  AIO167, AIO16A, QUAD8;

const
  FIFOHalfWords = 512;
  FIFOHalfBytes = 2 * FIFOHalfWords;
  NO_CARD = $FFFFFFFF;

type
  TAcqThread = class(TThread)
  public
    BytesLeftToTake: Integer;
    bReportQuad: Boolean;
  protected
    procedure Execute; override;
    procedure NoteThreadStop;
  end;

var
  AcqThread: TAcqThread = nil;
  QUADCardNum: LongWord;
  QUADBase: Word;
  ComNum: Integer;



procedure CtrMode(CardNum: LongWord; cntr, mode: Byte);
var
  ctrl: byte;

begin
  ctrl := (cntr shl 6) or $30 or (mode shl 1);
  RelOutPortB(CardNum, 8+3, ctrl);
end;

procedure CtrLoad(CardNum: LongWord; c, val: Word);
begin
  RelOutPortB(CardNum, 8+c, lo(val));
  RelOutPortB(CardNum, 8+c, hi(val));
end;



{ TSampleForm }

procedure TMainForm.FormCreate(Sender: TObject);
var
  I, NumCards: Integer;
  Status, Buf: LongWord;
  hCom: THandle;
  ComBuf: String;

begin
  AIO16CardNum := NO_CARD;
  QUADCardNum := NO_CARD;
  ComNum := 0;

  NumCards := GetNumCards;
  for I := 0 to NumCards - 1 do begin
    Status := QueryCardID(I, @Buf);
    if Status <> ERROR_SUCCESS then Continue; //Failed to query ID.
    case Buf of
      $1ECE8: if AIO16CardNum = NO_CARD then begin //104-AIO16E
        Status := QueryCardBase(I, @Buf);
        if Status <> ERROR_SUCCESS then Continue;

        AIO16CardNum := I;
      end;
      $12230: if QUADCardNum = NO_CARD then begin //104-QUAD-8
        Status := QueryCardBase(I, @QUADBase);
        if Status <> ERROR_SUCCESS then Continue;

        QUADCardNum := I;
      end;
      else Continue;
    end;

    if (AIO16CardNum <> NO_CARD) and (QUADCardNum <> NO_CARD) then Break;
  end;

  for I := 1 to 255 do begin
    ComBuf := '\\.\COM' + IntToStr(I);
    hCom := CreateFile(PChar(ComBuf), GENERIC_WRITE or GENERIC_READ, 0, nil, OPEN_EXISTING, 0, 0);
    ComBuf := '';
    if hCom <> INVALID_HANDLE_VALUE then begin
      ComNum := I;
      CloseHandle(hCom);
      Break;
    end;
  end;

  if AIO16CardNum = NO_CARD then
    LogMemo.Lines.Add('No 104-AIO16E found; make sure it''s installed in Device Manager.')
  else
    LogMemo.Lines.Add('104-AIO16E found as card number ' + IntToStr(AIO16CardNum) + '.')
  ;
  LogMemo.Lines.Add('');

  if QUADCardNum = NO_CARD then
    LogMemo.Lines.Add('No 104-QUAD-8 found; make sure it''s installed in Device Manager.')
  else
    LogMemo.Lines.Add('104-QUAD-8 found as card number ' + IntToStr(QUADCardNum) + '.')
  ;
  LogMemo.Lines.Add('');

  if ComNum = 0 then
    LogMemo.Lines.Add('No serial port found; make sure one is installed.')
  else
    LogMemo.Lines.Add('Serial port found at COM' + IntToStr(ComNum) + '.')
  ;
  LogMemo.Lines.Add('');

  if (AIO16CardNum <> NO_CARD) and (QUADCardNum <> NO_CARD) and (ComNum <> 0) then begin
    LogMemo.Lines.Add('Click Start to start acquisition.');
    LogMemo.Lines.Add('');
  end;
end;

procedure TMainForm.StartButtonClick(Sender: TObject);
begin
  if (AIO16CardNum = NO_CARD) or (QUADCardNum = NO_CARD) or (ComNum = 0) then begin
    Beep;
    LogMemo.Font.Color := $FFFFFF;
    LogMemo.Color := $0000FF;
    Exit;
  end;

  if AcqThread <> nil then begin
    Beep;
    Exit;
  end;



  //Reset entire board to clear out any previous config.
  RelOutPortB(AIO16CardNum, $1B, $FF);

  //Initialize the global variables and then load the card's calibration.
  init;
  doCAL;

  //tell the card to take one reading per channel using Scan Start
  RelOutPortB(AIO16CardNum, $11, $05);//Set ADC SCAN MODE and COUNTER START

  RelOutPortB(AIO16CardNum, $13, $02); //Enable IRQ on end-of-scan.

  //Both A/D counters in mode 2; they'll be loaded below.
  CtrMode(AIO16CardNum, 1, 2);
  CtrMode(AIO16CardNum, 2, 2);

  RelOutPortB(AIO16CardNum, $06, $F0); //scan all channels, zero to fifteen

  //Ctr 1 & 2 divisor of 166666 with 10MHz clock is close to 60Hz tick.
  CtrLoad(AIO16CardNum, 1, 167);
  CtrLoad(AIO16CardNum, 2, 998);



  //Init the quad board.
  Init_7266(QUADBase);



  //Launch the acquisition thread.
  AcqThread := TAcqThread.Create(True);
  AcqThread.bReportQuad := ReportRadios.ItemIndex <> 0;
  AcqThread.Resume;

  LogMemo.Lines.Add('Acquiring...');
end;

procedure TMainForm.StopButtonClick(Sender: TObject);
begin
  if AcqThread <> nil then begin
    LogMemo.Lines.Add('Stopping...');
    AcqThread.Terminate;
  end;
end;

procedure TMainForm.ReportRadiosClick(Sender: TObject);
begin
  if AcqThread <> nil then
    AcqThread.bReportQuad := ReportRadios.ItemIndex <> 0
  ;
end;

{ TAcqThread }

procedure TAcqThread.Execute;
const
  HexChar: array[0..15] of Char = ('0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F');

var
  ADData: array of Word;
  QuadData: array of LongInt;
  Buf: String;
  I: Integer;
  hCom: THandle;
  BytesTransferred: LongWord;
  ComDCB: TDCB;

begin
  ReturnValue := ERROR_SUCCESS;

  Buf := '\\.\COM' + IntToStr(ComNum);
  hCom := CreateFile(PChar(Buf), GENERIC_WRITE or GENERIC_READ, 0, nil, OPEN_EXISTING, 0, 0);

  SetupComm(hCom, 1026, 255); //In buffer = 1026 bytes, out buffer = 255 bytes
  GetCommState(hCom, ComDCB); //Fill ComDCB with current data
  ComDCB.BaudRate := 115200;
  ComDCB.ByteSize := 8;
  ComDCB.Parity   := NOPARITY;
  ComDCB.StopBits := ONESTOPBIT;
  SetCommState(hCom, ComDCB);

  SetLength(ADData, 16);
  SetLength(QuadData, 8);
  repeat
    ReturnValue := WaitForIRQData(AIO16CardNum, Length(ADData) * SizeOf(ADData[0]), @ADData[0]);
    if ReturnValue <> ERROR_SUCCESS then Break;

    for I := 0 to 7 do
      QuadData[I] := Read_7266_OL(QUADBase + I * 2)
    ;

    //We're only going at 60Hz, so we can do the string processing now.

    Buf := '';
    if bReportQuad then begin //This sample just reports one or the other.
      for I := 0 to 7 do begin
        Buf := Buf + IntToHex(QuadData[I], 6);
        Buf := Buf + ',';
      end;
    end
    else begin
      for I := 0 to 15 do begin
        Buf := Buf + IntToHex(ADData[I], 4);
        Buf := Buf + ',';
      end;
    end;
    Buf[Length(Buf)] := #13; //Change the last comma to an enter.

    WriteFile(hCom, Buf[1], Length(Buf), BytesTransferred, nil);
    Buf := '';
  until Terminated;

  //Stop the A/D counters by moding them.
  CtrMode(AIO16CardNum, 1, 2);
  CtrMode(AIO16CardNum, 2, 2);

  CloseHandle(hCom);

  Synchronize(NoteThreadStop);
end;

procedure TAcqThread.NoteThreadStop;
begin
  MainForm.LogMemo.Lines.Add('Stopped.');
  AcqThread := nil;
  FreeOnTerminate := True;
end;

end.
