unit QUAD8;

interface

procedure Init_7266(Addr: LongWord);
procedure Write_7266_PR(Addr: LongWord; Data: LongWord);
function Read_7266_OL(Addr: LongWord): integer;



implementation

uses
  AIO167;

const
  RLD = $80;
  XRLD = $00;
  YRLD = XRLD;
  Rst_BP = $01;
  Rst_CTR = $02;
  Rst_FLAGS = $04;
  Rst_E = $06;
  Trf_PR_CTR = $08;
  Trf_CTR_OL = $10;
  Trf_PS0_PSC = $18;

  CMR = $A0;
  XCMR = $20;
  YCMR = XCMR;
  BINCnt = $00;
  BCDCnt = $01;
  NrmCnt = $00;
  RngLmt = $02;
  NRcyc = $04;
  ModN = $06;
  NQDX = $00;
  QDX1 = $08;
  QDX2 = $10;
  QDX4 = $18;

  IOR = $C0;
  XIOR = $40;
  YIOR = $00;
  DisAB = $00;
  EnAB = $01;
  LCTR = $00;
  LOL = $02;
  RCTR = $00;
  ABGate = $04;
  CYBW = $00;
  CPBW = $08;
  CB_UPDN = $10;
  IDX_ERR = $18;

  IDR = $E0;
  XIDR = $60;
  YIDR = XIDR;
  DisIDX = $00;
  EnIDX = $01;
  NIDX = $00;
  PIDX = $02;
  LIDX = $00;
  RIDX = $04;

//---------------------------------------------------------------------------
procedure Init_7266(Addr: LongWord);
var
  XCt: LongWord;
  YCt: LongWord;
  count: Integer;

begin


    for count := 0 to 3 do
    begin
        XCt := $FFFFFFFF;
        YCt := $FFFFFFFF;

	    //Setup IOR reg
	    OutPortB(Addr + 1, IOR + DisAB + LOL + ABGate + CYBW);

    	//Setup RLD reg
	    OutPortB(Addr + 1, RLD + Rst_BP + Rst_FLAGS);
    	OutPortB(Addr + 0, $00);
	    OutPortB(Addr + 2, $00);
    	OutPortB(Addr + 1, RLD + Rst_E + Trf_PS0_PSC);
	    OutPortB(Addr + 1, RLD + Rst_BP + Rst_CTR);

    	//Setup IDR reg
	    OutPortB(Addr + 1, IDR + EnIDX + NIDX + LIDX);

    	//Setup CMR reg
	    OutPortB(Addr + 1, CMR + BINCnt + ModN + QDX1);

    	//Setup PR reg for modulo N counter to XCt
	    OutPortB(Addr + 0, XCt);
            XCt := XCt shr 8;
    	OutPortB(Addr + 0, XCt);
            XCt := XCt shr 8;
    	OutPortB(Addr + 0, XCt);

	    //Setup PR reg for modulo N counter to YCt
    	OutPortB(Addr + 2, YCt);
            YCt := YCt shr 8;
    	OutPortB(Addr + 2, YCt);
            YCt := YCt shr 8;
    	OutPortB(Addr + 2, YCt);

	    //Enable counters
    	OutPortB(Addr + 1, IOR + EnAB);

        inc(Addr, 4);
    end;
end;

procedure Write_7266_PR(Addr: LongWord; Data: LongWord);
begin
	OutPortB(Addr + 1, RLD + Rst_BP);
	OutPortB(Addr + 0, Data);
	Data := Data shr 8;
	OutPortB(Addr + 0, Data);
	Data := Data shr 8;
	OutPortB(Addr + 0, Data);
end;

function Read_7266_OL(Addr: LongWord): integer;
var
  reading: array[0..2] of BYTE;

begin
  OutPortB(ADDR + 1, RLD + Rst_BP + Trf_CTR_OL);

  reading[0] := InPortB(Addr);
  reading[1] := InPortB(Addr);
  reading[2] := InPortB(Addr);

  if (reading[2] and $80) <> 0 then
  begin
    reading[2] := not reading[2];
    reading[1] := not reading[1];
    reading[0] := not reading[0];
    result := (reading[2] shl 16) or (reading[1] shl 8) or reading[0];
    result := (result * -1) - 1;
  end
  else
    result := (reading[2] shl 16) or (reading[1] shl 8) or reading[0]
  ;
end;

function Get_7266_Flags(Addr: LongWord) : BYTE;
begin
  Get_7266_Flags :=  InPortB(Addr + 1);
end;

//---------------------------------------------------------------------------

end.
